{
  open Parser

  exception Eof_in_comment

  let incr_linenum lexbuf =
    let pos = lexbuf.Lexing.lex_curr_p in
      lexbuf.Lexing.lex_curr_p <- {
	pos with
	  Lexing.pos_lnum = pos.Lexing.pos_lnum + 1;
	  Lexing.pos_bol = pos.Lexing.pos_cnum;
      }
}

let integer = ['0'-'9']+

let id = ['a'-'z' 'A'-'Z'] ['a'-'z' 'A'-'Z' '0'-'9']*

rule token = parse
  | "letrec" { TOLETREC }
  | '+' { TOPLUS }
  | '-' { TOMINUS }
  | '/' { TODIV }
  | '*' { TOMULT }
  | '%' { TOMOD }
  | "<=" { TOLEQ }
  | ">=" { TOGEQ }
  | "<" { TOLESS }
  | ">" { TOGREATER }
  | "let" { TOLET }
  | "in" { TOIN }
  | "true" | "false" as boolean { TOBOOL(bool_of_string boolean) }
  | "and" { TOAND }
  | "not" { TONOT }
  | "or" { TOOR }
  | "xor" { TOXOR }
  | '=' { TOEQUAL }
  | "if" { TOIF }
  | "then" { TOTHEN }
  | "else" { TOELSE }
  | "fun" { TOFUN }
  | "ref" { TOREF }
  | "!" { TOBANG }
  | ":=" { TOASSIGN }
  | ";" { TODOUBLECOLON }
  | "()" { TOUNIT }
  | "(" { TOLPAREN }
  | ")" { TORPAREN }
  | "type" { TOTYPE }
  | "in" { TOIN }
  | integer as number { TOINT(Big_int.big_int_of_string number) }
  | id as str { TOID(str) } 
  | [' ' '\t'] { token lexbuf }
  | ['\n'] { incr_linenum lexbuf; token lexbuf }
  | "//" { commentseol lexbuf }
  | "/*" { comments 0 lexbuf }
  | eof { TOEOF }

and comments level = parse
  | "*/" { if level = 0 then token lexbuf else comments (level - 1) lexbuf }
  | "/*" { comments (level + 1) lexbuf }
  | _ { comments level lexbuf }
  | eof { raise Eof_in_comment }

and commentseol = parse
  | '\n' { token lexbuf }
  | _ { commentseol lexbuf }
  | eof { TOEOF }
