type pexpr = Lexing.position * expr
and expr =
    (* the standard lambda calculus *)
  | VAR of string
  | APP of pexpr * pexpr
  | FUN of string * pexpr
      
  (* standard binding operators *)
  | LET of string * pexpr * pexpr
  | LETREC of string * pexpr * pexpr

  (* the "unit" value *)
  | UNIT

  (* integers and operations on integers *)
  | INT of Big_int.big_int
  | UMINUS of pexpr
  | PLUS of pexpr * pexpr
  | MINUS of pexpr * pexpr
  | MULT of pexpr * pexpr
  | DIV of pexpr * pexpr
  | MOD of pexpr * pexpr
  | GREATER of pexpr * pexpr
  | LESS of pexpr * pexpr
  | LEQ of pexpr * pexpr
  | GEQ of pexpr * pexpr

  (* booleans and operations on booleans *)
  | BOOL of bool
  | NOT of pexpr
  | AND of pexpr * pexpr
  | OR of pexpr * pexpr
  | XOR of pexpr * pexpr
  | ITE of pexpr * pexpr * pexpr

  (* equality can compare booleans, integers and units *)
  | EQUAL of pexpr * pexpr
  
  (* pointers *)
  | REF of pexpr
  | BANG of pexpr
  | ASSIGN of pexpr * pexpr

  (* sequence of expressions *)
  | SEQ of pexpr * pexpr
  
  (* a begin-end block *)
  | BLOCK of pexpr

  (* dummy syntax -- TYPE(pexpr) simpy evaluates pexpr. The type
  adnotation is to tell the typer to output the principal type of this
  expression *)
  | TYPE of pexpr

