%{
  open Mlsyntax
%}

%token <Big_int.big_int> TOINT
%token <bool> TOBOOL
%token <string> TOID
%token TOUNIT

%token TOTYPE
%token TOFUN
%nonassoc TOFUN
%token TOLET
%nonassoc TOLET
%token TOLETREC
%nonassoc TOLETREC

%token TODOUBLECOLON
%left TODOUBLECOLON

%token TOASSIGN
%right TOASSIGN
%token TOREF
%nonassoc TOREF
%token TONOT
%nonassoc TONOT
%token TOAND TOOR TOXOR
%right TOAND TOOR TOXOR
%token TOGREATER TOLESS TOLEQ TOGEQ
%nonassoc TOGREATER TOLESS TOLEQ TOGEQ
%token TOEQUAL
%nonassoc TOEQUAL
%nonassoc TONEG
%token TOPLUS TOMINUS
%left TOPLUS TOMINUS
%token TOMULT TODIV TOMOD
%left TOMULT TODIV TOMOD
%token TOBANG
%nonassoc TOBANG

%token TOIF TOTHEN TOELSE

%token TOEOF
%token TOIN

%token TOLPAREN TORPAREN

%start input
%type <Mlsyntax.pexpr> input e

%%

  e:
| e1 { $1 }
| TOFUN TOID TOEQUAL e %prec TOFUN { (Parsing.symbol_start_pos (), FUN($2, $4)) }
| TOIF e TOTHEN e TOELSE e { (Parsing.symbol_start_pos (), ITE($2, $4, $6)) }
| TOLET TOID TOEQUAL e TOIN e %prec TOLET { (Parsing.symbol_start_pos (), LET($2, $4, $6)) }
| TOLETREC TOID TOEQUAL e TOIN e %prec TOLETREC { (Parsing.symbol_start_pos (), LETREC($2, $4, $6)) }
| e TOASSIGN e { (Parsing.symbol_start_pos (), ASSIGN($1, $3)) }
| e TODOUBLECOLON e { (Parsing.symbol_start_pos (), SEQ($1, $3)) }
| e TOEQUAL e { (Parsing.symbol_start_pos (), EQUAL($1, $3)) } 
| e TOPLUS e { (Parsing.symbol_start_pos (), PLUS($1, $3)) }
| e TOMINUS e { (Parsing.symbol_start_pos (), MINUS($1, $3)) }
| TOMINUS e %prec TONEG { (Parsing.symbol_start_pos (), UMINUS($2)) }
| e TOMULT e { (Parsing.symbol_start_pos (), MULT($1, $3)) }
| e TODIV e { (Parsing.symbol_start_pos (), DIV($1, $3)) }
| e TOMOD e { (Parsing.symbol_start_pos (), MOD($1, $3)) }
| e TOGREATER e { (Parsing.symbol_start_pos (), GREATER($1, $3)) }
| e TOLESS e { (Parsing.symbol_start_pos (), LESS($1, $3)) }
| e TOLEQ e { (Parsing.symbol_start_pos (), LEQ($1, $3)) }
| e TOGEQ e { (Parsing.symbol_start_pos (), GEQ($1, $3)) }
| e TOAND e { (Parsing.symbol_start_pos (), AND($1, $3)) }
| e TOOR e { (Parsing.symbol_start_pos (), OR($1, $3)) }
| e TOXOR e { (Parsing.symbol_start_pos (), XOR($1, $3)) }
  ;
  
  e1:
| e1 closedexpr { (Parsing.symbol_start_pos (), APP($1, $2)) }
| closedexpr { $1 }
  ;
  
  closedexpr:
| TOBANG e { (Parsing.symbol_start_pos (), BANG($2)) }
| TOID { (Parsing.symbol_start_pos (), VAR($1)) }
| TOUNIT { (Parsing.symbol_start_pos (), UNIT) }
| TOINT { (Parsing.symbol_start_pos (), INT($1)) }
| TOLPAREN e TORPAREN { (Parsing.symbol_start_pos (), BLOCK($2)) }
| TOBOOL { (Parsing.symbol_start_pos (), BOOL($1)) }
| TONOT e { (Parsing.symbol_start_pos (), NOT($2)) }
| TOREF e { (Parsing.symbol_start_pos (), REF($2)) }
| TOTYPE TOLPAREN e TORPAREN { (Parsing.symbol_start_pos (), TYPE($3)) }
  ;  
  
  input: e TOEOF { $1 }
  ;
